@@ -817,6 +817,13 @@ gpgrt_free (void *a)
     _gpgrt_free (a);
 }
 
+void
+gpgrt_wipememory (void *ptr, size_t len)
+{
+  if (ptr && len)
+    _gpgrt_wipememory (ptr, len);
+}
+
 char *
 gpgrt_getenv (const char *name)
 {
@@ -954,6 +961,14 @@ gpgrt_log_get_stream (void)
   return _gpgrt_log_get_stream ();
 }
 
+
+void
+gpgrt_add_post_log_func (void (*f)(int))
+{
+  _gpgrt_add_post_log_func (f);
+}
+
+
 void
 gpgrt_log (int level, const char *fmt, ...)
 {
@@ -977,6 +992,17 @@ gpgrt_logv_prefix (int level, const char *prefix,
   _gpgrt_logv_prefix (level, prefix, fmt, arg_ptr);
 }
 
+void
+gpgrt_logv_domain (const char *domain, int level, const char *prefix,
+                   const void *buffer, size_t length,
+                   const char *fmt, va_list arg_ptr)
+{
+  (void)domain;  /* Not yet used.  */
+  (void)buffer;
+  (void)length;
+  _gpgrt_logv_prefix (level, prefix, fmt, arg_ptr);
+}
+
 void
 gpgrt_log_string (int level, const char *string)
 {
